// ============================================================================
// Project:	Deductive Database
// Filename:	atom.h
// Purpose:	Atoms (Symbolic Constants, Enumeration Type Constants)
// Last Change:	07.09.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	ATOM_INCLUDED
#define ATOM_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// ATOM_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long ATOM_MAGIC = 0x41544F0AL;	// 'ATO\n'


//=============================================================================
// Class for Atoms (Symbolic Constants, Enumeration Type Constants)
//=============================================================================

class atom_c {
	public:


//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor (Not implemented, use init() instead, see atomtab_c):
	atom_c();

//	// Destructor:
//	~atom_c();


//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// init: Used in place of the constructor.
	void init(str_t name_start, str_t name_end, int atom_id, atom_c *next);
		// atom_id is returned by id()
		// next is returned by hash_next()
		// gcc complained about shadowed names.

	// name: Return name of atom.
	str_t name() {
		CHECK_VALID("atom_c::name");
		return Name;
	}

	// id: Return number of this atom (unique within one atomtab_c).
	int id() {
		CHECK_VALID("atom_c::id");
		return Id;
	}

	// hash_next: Return next atom in hash chain.
	atom_c* hash_next() {
		CHECK_VALID("atom_c::hash_next");
		return HashNext;
	}


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "ATOM_MAGIC".
#endif


//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	atom_c(const atom_c& atom);		// Not implemented
	atom_c& operator=(const atom_c& atom);	// Not implemented



//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// Name of this atom:
	char Name[VER_MAX_ATOMSIZE+1];

	// Number of this atom (will be unique within one atomtab_c):
	int Id;

	// Link to next atom in hash chain (see class atomtab_c).
	atom_c *HashNext;


//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef atom_c *atom_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define ATOM_NULL (static_cast<atom_t>(0))

//=============================================================================
// End of Include File:
//=============================================================================

#endif

